import argparse
import cv2 as cv
from TwitterGraph.manager import GraphManager
from TwitterGraph.multigraph import MultiG

def cli():
    parser = argparse.ArgumentParser(description='Twitter-Graph')
    parser.add_argument('--victims', type=str, help='String de nombre o nombres de víctimas')
    parser.add_argument('--n_tweets', type=int, help='Número de tweets a scrappear')
    parser.add_argument('--depth', type=int, help='Profundidad del scrapeo')

    args = parser.parse_args()
    
    if not '-' in args.victims:
        manager = GraphManager(args.victims, args.n_tweets, args.depth)
        img = cv.imread(manager.dir+manager.path[:-4]+'.png')
    else: 
        manager = MultiG(args.victims.split('-'), args.n_tweets)
        img = cv.imread(manager.path[:-4]+'.png')
    
    if img is None:
        exit(1)

    cv.imshow("Display window", img)
    cv.waitKey(0)


import snscrape.modules.twitter as sntwitter
import pandas as pd


class DataScrapper():

    def __init__(self, victim='', n_twitts=100) -> None:
        self.victim   = victim
        self.n_twitts = n_twitts if n_twitts else 1

    def get_query(self, query, limit=None):
        if limit is None:
            limit = self.n_twitts
        results = []
        for j, tweet in enumerate(sntwitter.TwitterSearchScraper(query).get_items()):
            if j > limit:
                break
            results.append([tweet.user.username, tweet.date, tweet.replyCount, tweet.retweetCount, tweet.likeCount,
                            tweet.inReplyToUser, tweet.mentionedUsers, tweet.rawContent])
        return pd.DataFrame(results, columns=['Username', 'Date', 'Replies','RTs', 'Likes', 'Reply_to', 'Mentions', 'Tweets'])
    
    
    def are_mentions(self, user_from, user_to):
        results = []
        for tw in sntwitter.TwitterSearchScraper(f'from:{user_from} to:{user_to}').get_items():
            if len(results) > 0:
                break
            results.append(tw)
        return len(results) > 0


    # devuelve las últimas menciones a victim - sin tener en cuenta el emisor
    def get_all_mentions(self, limit=None):
        # respuestas directas y menciones
        q1 = f'to:{self.victim}'
        q2 = f'@{self.victim}'
        return pd.concat([self.get_query(q1, limit), self.get_query(q2, limit)], axis=0)

    # get_mentions_from -> devuelve los tweets donde user_from responde o menciona a self.victim

    def get_mentions_from(self, user_from, limit=None):
        # respuestas directas y menciones
        q1 = f'from:{user_from} to:{self.victim}'
        q2 = f'@{self.victim} from:{user_from}'
        return pd.concat([self.get_query(q1, limit), self.get_query(q2, limit)], axis=0)

    # get_mentions_to -> devuelve los tweets donde self.victim menciona o responde a user_from
    def get_mentions_to(self, user_to, limit=None):
        # menciones directas y laterales
        q1 = f'from:{self.victim} to:{user_to}'
        q2 = f'@{user_to} from:{self.victim}'
        return pd.concat([self.get_query(q1, limit), self.get_query(q2, limit)], axis=0)

    # get_user -> Devuelve instancia User (snstwitter class)
    def get_user(self, usr):
        user = sntwitter.TwitterUserScraper(usr).entity

    # SCRAPPING LAST TWEETS OF AND TO VICTIM
    # get_inputs -> devuelve los últimos tweets dirigidos a self.victim
    def get_inputs(self, limit=None):
        q = f'to:{self.victim}'
        return self.get_query(q, limit)

    # get_outputs -> devuelve los últimos n_twitts de self.victim
    def get_outputs(self):
        q = f'from:{self.victim}'
        return self.get_query(q)



from TwitterGraph.datascrapper import DataScrapper
from networkx import strongly_connected_components as scc
from datascrapper import DataScrapper
import networkx as nx


class Tree():

    def __init__(self, user_0, batch_=10, depth_=1) -> None:
        self.root = user_0
        self.batch = batch_
        self.depth = depth_ if depth_ else -1
        self.path = f'tweets_{self.batch}_depth_{self.depth}'
        self.nx = nx.DiGraph()
        self.nx.add_node(self.root, color='red')
        self.build()
        self.add_missing_edges()
    

    def add_edge(self, node_from, node_to):
        if self.root == node_from == node_to:
            col = 'green'
        elif self.root == node_from:
            col = 'red'
        else:
            col = 'blue' if self.root == node_to else 'black'
        # creamos la arista
        self.nx.add_edge(node_from, node_to, color=col)
        

    def build(self, user_=None, batch_=None, depth_=None) -> None:

        # construye recursivamente el grafo de reaciones empezando en el nodo root
        if user_ is None:
            user_ = self.root
            batch_ = self.batch
            depth_ = self.depth

        # scrap los últimos tweets outputs de "user"
        replies = set()
        last_tweets = DataScrapper(user_, batch_).get_outputs()
        for _, row in last_tweets.iterrows():
            if row['Reply_to'] is not None:
                usr = row['Reply_to'].username.lower()
                replies.add(usr)
            if row['Mentions'] is not None:
                for m in row['Mentions']:
                    if m is not None:
                        usr = m.username.lower()
                        replies.add(usr)

        # recorrer las respuestas encontradas
        for child in replies:
            self.add_edge(user_, child)
            if depth_ > 0:
                # llamada recursiva
                self.build(child, max(batch_//2, 0), depth_-1)

    # add_missing_edges -> añadimos comunicaciones con la víctima no reflejadas de nodos presentes en el grafo
    def add_missing_edges(self):
        # verificamos si algún nodo tiene conexión con root
        for n in self.nx.nodes:
            if n != self.root:
                if not self.nx.has_edge(self.root, n) and not DataScrapper().are_mentions(self.root,n):
                    self.add_edge(self.root, n)

                if not self.nx.has_edge(n, self.root) and not DataScrapper().are_mentions(n,self.root):
                    self.add_edge(n, self.root)



    # TODO: REVISAR - NOT USED!!
    def clean_tree(self):
        # limpia los nodos del árbol con input_deg <= 1 si no son adyacentes a victim
        to_remove = [n for n in self.nx.nodes if self.nx.in_degree(
            n) <= 1 and not self.nx.has_edge(self.root, n)]
        self.nx.remove_nodes_from(to_remove)

    def get_shortest_path(self, nodo_1, nodo_2):
        assert (nodo_1 in self.nx.nodes) and (nodo_2 in self.nx.nodes)
        return nx.dijkstra_path(self.nx, nodo_1, nodo_2)

    def get_distance(self, nodo_1, nodo_2):
        return len(self.get_shortest_path(nodo_1, nodo_2))



from manager import GraphManager
import networkx as nx
from os import path
import os

# politica españa:
# derecha : vox_es, santi_abascal, macarena_olona, ivanedlm
# izquierda: irenemontero, podemos, agarzon, pabloiglesias, sanchezcastejon

# política eeuu:
# derecha: realdonaldtrump, speakermccarthy, repmikegarcia
# izquierda: joebiden, barackobama, hillaryclinton

# actores eeuu:
# aaronpaul_8, bryancranston, breakingbad


class MultiG:

    def __init__(self, victims, n_tweets=10, depth_=1, ruta=None) -> None:

        self.victims = ('_').join(victims)
        self.n_tweets = n_tweets
        self.depth = depth_ if depth_ else 1
        self.graph = nx.DiGraph()

        # RUTA - CARPETA Y ARCHIVO
        if ruta is None:
            self.path = f'{os.getcwd()}/resources/multi/' + ('_').join(victims)
            self.path += f'/tweets_{self.n_tweets}_depth_{self.depth}'
        else:
            self.path = ruta
        # SI EXISTE EL ARCHIVO - SE CARGA
        if path.exists(self.path + '.dot'):
            self.manager = GraphManager('', -1, -1, self.path + '.dot')
            if not path.exists(self.path + '.png'):
                self.manager.save(self.path)
                self.manager.save_friends(self.path)
        else:
            # NO EXISTE EL ARCHIVO
            for v in victims:
                # se calcula y se guarda el árbol de forma ind para cada victima
                man = GraphManager(v, n_tweets, depth_)
                self.graph = nx.compose(self.graph, nx.DiGraph(man.nx))
            
            if not path.exists(f'{os.getcwd()}/resources/multi'):
                # Create a new directory because it does not exist
                os.makedirs(f'{os.getcwd()}/resources/multi')
                
            if not path.exists(f'{os.getcwd()}/resources/multi/' + ('_').join(victims)):
                os.makedirs(f'{os.getcwd()}/resources/multi/' + ('_').join(victims))
                
            nx.nx_pydot.write_dot(self.graph, self.path + '.dot')
            self.manager = GraphManager('', -1, -1, self.path + '.dot')
            self.manager.save(self.path + '.dot')
            self.manager.save_friends(self.path + '_amigos.dot')


from os import path
from tree import Tree
import networkx as nx
import time
import os


class GraphManager:

    def __init__(self, victim_, n_tweets=10, depth_=1, ruta=None) -> None:
        self.victim = victim_
        self.n_tweets = n_tweets if n_tweets is not None else 10
        self.depth = depth_ if depth_ is not None else 1

        # RUTA - CARPETA Y ARCHIVO
        if ruta is None:
            self.dir  = f'{os.getcwd()}/resources/victims/{self.victim}'
            self.path = f'/tweets_{self.n_tweets}_depth_{self.depth}.dot'
        else:
            self.path, self.dir = ruta, ''
        
        # SI EXISTE EL ARCHIVO - SE CARGA
        if path.exists(self.dir + self.path) and path.exists(self.dir + self.path[:-4]+'.png'):
            self.nx = nx.nx_pydot.read_dot(self.dir + self.path)
        elif path.exists(self.dir + self.path ) and not path.exists(self.dir + self.path[:-4]+'.png'):
            self.nx = nx.nx_pydot.read_dot(self.dir + self.path)
            self.save()
            self.save_friends()
        else:
            # SI NO EXISTE EL DIRECTORIO - SE CREA
            if self.dir != '' and not os.path.exists(self.dir):
                os.makedirs(self.dir, mode=0o777)
            # SE CALCULA EL GRAFO Y SE GUARDA
            t = Tree(self.victim, self.n_tweets, self.depth)
            self.nx = t.nx
            self.save()
            self.save_friends()
    
    # save -> save full graph in .dot and .png
    def save(self, s=None):
        s = (self.dir+self.path)[:-4] if s is None else s
        nx.nx_pydot.write_dot(self.nx, s + '.dot')   # SAVE .dot
        g = nx.nx_agraph.to_agraph(self.nx)
        g.draw(s + '.png', prog='circo') # SAVE .png
    
    # save_friends -> save bidir/friends to .png
    def save_friends(self, s=None):
        s = (self.dir+self.path)[:-4] if s is None else s
        if self.victim:
            amigos = self.get_friends() + [self.victim]
        else:
            amigos = self.get_bidir()
        nx.nx_pydot.write_dot(self.nx,s+'_amigos.png')
        g = nx.nx_agraph.to_agraph(self.nx.subgraph(amigos))
        g.draw(path=s+'_amigos.png', prog='circo')


    #get_friends -> devuelve nodos 'amigos' - aquellos con los que víctima tiene comunicación bidireccional
    def get_friends(self):
        return [n for n in self.nx.nodes if n != self.victim and self.nx.has_edge(self.victim, n)
               and self.nx.has_edge(n, self.victim)]

    # get bidir -> devuelve los nodos del grafo que tienen alguna comunicación bidireccional entre sí
    def get_bidir(self):
        bidir = []
        nodes = list(self.nx.nodes)
        for n1 in range(len(nodes)-1):
            for n2 in range(n1+1, len(nodes)):
                if self.nx.has_edge(nodes[n1], nodes[n2]) and self.nx.has_edge(nodes[n2], nodes[n1]):
                    bidir += [nodes[n1]] + [nodes[n2]]
        return list(set(bidir))
